@@ -1,19 +1,19 @@
 from __future__ import absolute_import, division, print_function
 
-import bcolz
 import os
 from bcolz import ctable, carray
 import numpy as np
 from toolz import keyfilter
 import datashape
 from datashape import discover
+import shutil
 from ..append import append
 from ..convert import convert, ooc_types
-from ..create import create
 from ..resource import resource
-from ..chunks import chunks, Chunks
+from ..drop import drop
+from ..chunks import chunks
 
-keywords = ['rootdir']
+keywords = ['cparams', 'dflt', 'expectedlen', 'chunklen', 'rootdir']
 
 
 @discover.register((ctable, carray))
@@ -30,7 +30,7 @@ def numpy_append_to_bcolz(a, b, **kwargs):
 
 @append.register((ctable, carray), object)
 def numpy_append_to_bcolz(a, b, **kwargs):
-    return append(a, convert(np.ndarray, b, **kwargs), **kwargs)
+    return append(a, convert(chunks(np.ndarray), b, **kwargs), **kwargs)
 
 
 @convert.register(ctable, np.ndarray, cost=2.0)
@@ -68,22 +68,43 @@ def bcolz_to_numpy_chunks(x, chunksize=2**20, **kwargs):
 
 
 @resource.register('.*\.bcolz/?')
-def resource_bcolz(uri, dshape=None, **kwargs):
+def resource_bcolz(uri, dshape=None, expected_dshape=None, **kwargs):
     if os.path.exists(uri):
-        return ctable(rootdir=uri)
+        try:
+            return ctable(rootdir=uri)
+        except IOError:  # __rootdirs__ doesn't exist because we aren't a ctable
+            return carray(rootdir=uri)
     else:
         if not dshape:
             raise ValueError("Must specify either existing bcolz directory or"
-                    "valid datashape")
+                             " valid datashape")
         dshape = datashape.dshape(dshape)
 
         dt = datashape.to_numpy_dtype(dshape)
-        x = np.empty(shape=(0,), dtype=dt)
+        shape_tail = tuple(map(int, dshape.shape[1:]))  # tail of shape
+        if dshape.shape[0] == datashape.var:
+            shape = (0,) + shape_tail
+        else:
+            shape = (int(dshape.shape[0]),) + shape_tail
+
+        x = np.empty(shape=shape, dtype=dt)
+
+        kwargs = keyfilter(keywords.__contains__, kwargs)
+        expectedlen = kwargs.pop('expectedlen',
+                                 int(expected_dshape[0])
+                                 if expected_dshape is not None and
+                                 isinstance(expected_dshape[0], datashape.Fixed)
+                                 else None)
 
         if datashape.predicates.isrecord(dshape.measure):
-            return ctable(x, rootdir=uri, **keyfilter(keywords.__contains__, kwargs))
+            return ctable(x, rootdir=uri, expectedlen=expectedlen, **kwargs)
         else:
-            return carray(x, rootdir=uri, **keyfilter(keywords.__contains__, kwargs))
+            return carray(x, rootdir=uri, expectedlen=expectedlen, **kwargs)
+
+
+@drop.register((carray, ctable))
+def drop_bcolz(b, **kwargs):
+    shutil.rmtree(b.rootdir)
 
 
 ooc_types |= set((carray, ctable))
