@@ -4,7 +4,7 @@ import csv
 
 import re
 import datashape
-from datashape import discover
+from datashape import discover, Record, Option
 from datashape.predicates import isrecord
 from datashape.dispatch import dispatch
 from collections import Iterator
@@ -14,12 +14,15 @@ import pandas as pd
 import os
 import gzip
 import bz2
+import uuid
 
+from ..compatibility import unicode
 from ..utils import keywords
 from ..append import append
 from ..convert import convert, ooc_types
 from ..resource import resource
 from ..chunks import chunks
+from ..temp import Temp
 from ..numpy_dtype import dshape_to_pandas
 from .pandas import coerce_datetimes
 
@@ -104,7 +107,7 @@ def ext(path):
     return e.lstrip('.')
 
 
-@convert.register(pd.DataFrame, CSV, cost=20.0)
+@convert.register(pd.DataFrame, (Temp(CSV), CSV), cost=20.0)
 def csv_to_DataFrame(c, dshape=None, chunksize=None, nrows=None, **kwargs):
     try:
         return _csv_to_DataFrame(c, dshape=dshape,
@@ -151,7 +154,7 @@ def _csv_to_DataFrame(c, dshape=None, chunksize=None, **kwargs):
         except StopIteration:
             found_names = pd.read_csv(c.path, encoding=encoding,
                                       compression=compression)
-
+    if names and header == 'infer':
         if [n.strip() for n in found_names] == [n.strip() for n in names]:
             header = 0
         elif (all(re.match('^\s*\D\w*\s*$', n) for n in found_names) and
@@ -174,7 +177,7 @@ def _csv_to_DataFrame(c, dshape=None, chunksize=None, **kwargs):
                              **kwargs2)
 
 
-@convert.register(chunks(pd.DataFrame), CSV, cost=10.0)
+@convert.register(chunks(pd.DataFrame), (Temp(CSV), CSV), cost=10.0)
 def CSV_to_chunks_of_dataframes(c, chunksize=2**20, **kwargs):
     # Load a small 1000 line DF to start
     # This helps with rapid viewing of a large CSV file
@@ -191,8 +194,8 @@ def CSV_to_chunks_of_dataframes(c, chunksize=2**20, **kwargs):
 
 
 @discover.register(CSV)
-def discover_csv(c):
-    df = csv_to_DataFrame(c, nrows=50)
+def discover_csv(c, nrows=1000, **kwargs):
+    df = csv_to_DataFrame(c, nrows=nrows, **kwargs)
     df = coerce_datetimes(df)
 
     if (not list(df.columns) == list(range(len(df.columns)))
@@ -200,14 +203,22 @@ def discover_csv(c):
         df = csv_to_DataFrame(c, chunksize=50, has_header=False).get_chunk()
         df = coerce_datetimes(df)
 
-    df.columns = [str(c).strip() for c in df.columns]
+    columns = [str(c) if not isinstance(c, (str, unicode)) else c
+                for c in df.columns]
+    df.columns = [c.strip() for c in columns]
 
     # Replace np.nan with None.  Forces type string rather than flaot
     for col in df.columns:
         if df[col].count() == 0:
             df[col] = [None] * len(df)
 
-    return datashape.var * discover(df).subshape[0]
+    measure = discover(df).measure
+
+    # Use Series.notnull to determine Option-ness
+    measure2 = Record([[name, Option(typ) if (~df[name].notnull()).any() else typ]
+                      for name, typ in zip(measure.names, measure.types)])
+
+    return datashape.var * measure2
 
 
 @resource.register('.+\.(csv|tsv|ssv|data|dat)(\.gz|\.bz)?')
@@ -228,48 +239,23 @@ def resource_glob(uri, **kwargs):
     return chunks(type(r))(_)
 
 
-@convert.register(chunks(pd.DataFrame), chunks(CSV), cost=10.0)
+@convert.register(chunks(pd.DataFrame), (chunks(CSV), chunks(Temp(CSV))), cost=10.0)
 def convert_glob_of_csvs_to_chunks_of_dataframes(csvs, **kwargs):
     def _():
         return concat(convert(chunks(pd.DataFrame), csv, **kwargs) for csv in csvs)
     return chunks(pd.DataFrame)(_)
 
 
+@convert.register(Temp(CSV), (pd.DataFrame, chunks(pd.DataFrame)))
+def convert_dataframes_to_temporary_csv(data, **kwargs):
+    fn = '.%s.csv' % uuid.uuid1()
+    csv = Temp(CSV)(fn, **kwargs)
+    return append(csv, data, **kwargs)
+
+
 @dispatch(CSV)
 def drop(c):
     os.unlink(c.path)
 
 
-"""
-@append.register(CSV, Iterator)
-def append_iterator_to_csv(c, seq, dshape=None, **kwargs):
-    f = open(c.path, mode='a')
-
-    # TODO: handle dict case
-    writer = csv.writer(f, **c.dialect)
-    writer.writerows(seq)
-    return c
-
-
-try:
-    from dynd import nd
-    @convert.register(Iterator, CSV)
-    def csv_to_iterator(c, chunksize=1024, dshape=None, **kwargs):
-        f = open(c.path)
-        reader = csv.reader(f, **c.dialect)
-        if c.header:
-            next(reader)
-
-        # Launder types through DyND
-        seq = pipe(reader, partition_all(chunksize),
-                           map(partial(nd.array, type=str(dshape.measure))),
-                           map(partial(nd.as_py, tuple=True)),
-                           concat)
-
-        return reader
-except ImportError:
-    pass
-"""
-
-
 ooc_types.add(CSV)
