@@ -5,12 +5,14 @@ import os
 import pandas as pd
 import gzip
 import datashape
+from datashape import Option, string
 from collections import Iterator
 
 from into.backends.csv import (CSV, append, convert, resource,
         csv_to_DataFrame, CSV_to_chunks_of_dataframes)
 from into.utils import tmpfile, filetext, filetexts, raises
-from into import into, append, convert, resource, discover, dshape
+from into import into, append, convert, resource, discover, dshape, Temp
+from into.temp import _Temp
 from into.compatibility import unicode, skipif
 
 
@@ -161,7 +163,7 @@ def test_discover_csv_files_without_header():
 
 
 def test_discover_csv_yields_string_on_totally_empty_columns():
-    expected = dshape('var * {a: int64, b: string, c: int64}')
+    expected = dshape('var * {a: int64, b: ?string, c: int64}')
     with filetext('a,b,c\n1,,3\n4,,6\n7,,9') as fn:
         csv = CSV(fn, has_header=True)
         assert discover(csv) == expected
@@ -183,7 +185,6 @@ def test_pandas_csv_naive_behavior_results_in_columns():
                        [4, 'Denis',   400],
                        [5, 'Edith',  -500]], columns=['id', 'name', 'amount'])
     with tmpfile('.csv') as fn:
-        os.remove(fn)
         into(fn, df)
 
         with open(fn) as f:
@@ -251,3 +252,47 @@ def test_empty_dataframe():
         csv = CSV(fn, has_header=True)
         df = convert(pd.DataFrame, csv)
         assert isinstance(df, pd.DataFrame)
+
+
+def test_csv_missing_values():
+    with filetext('name,val\nAlice,100\nNA,200', extension='csv') as fn:
+        csv = CSV(fn)
+        assert discover(csv).measure.dict['name'] == Option(string)
+
+
+def test_csv_separator_header():
+    with filetext('a|b|c\n1|2|3\n4|5|6', extension='csv') as fn:
+        csv = CSV(fn, delimiter='|', has_header=True)
+        assert convert(list, csv) == [(1, 2, 3), (4, 5, 6)]
+
+
+df = pd.DataFrame([['Alice',   100],
+                   ['Bob',     200],
+                   ['Charlie', 300]],
+                  columns=['name', 'balance'])
+
+
+def test_temp_csv():
+    csv = into(Temp(CSV)('_test_temp_csv.csv'), df)
+    assert isinstance(csv, CSV)
+
+    assert into(list, csv) == into(list, df)
+
+    del csv
+    import gc
+    gc.collect()
+    assert not os.path.exists('_test_temp_csv.csv')
+
+
+def test_convert_to_csv():
+    csv = into(Temp(CSV), df)
+    assert isinstance(csv, CSV)
+
+    assert into(list, csv) == into(list, df)
+    assert isinstance(csv, _Temp)
+
+
+def test_unicode_column_names():
+    with filetext('foo\xc4\x87,a\n1,2\n3,4', extension='csv') as fn:
+        csv = CSV(fn, has_header=True)
+        df = into(pd.DataFrame, csv)
