@@ -1,5 +1,8 @@
 from __future__ import absolute_import, division, print_function
 
+import pytest
+pytest.importorskip('dynd')
+
 from into.backends.dynd import convert, append, create, discover
 from into import into, append, convert, resource, discover
 from dynd import nd
