@@ -9,19 +9,12 @@ import datashape
 from .core import NetworkDispatcher, ooc_types
 from .chunks import chunks, Chunks
 from .numpy_dtype import dshape_to_numpy
+from .utils import records_to_tuples
 
 
 convert = NetworkDispatcher('convert')
 
 
-def identity(x, **kwargs):
-    """
-
-    >>> identity('anything')
-    'anything'
-    """
-    return x
-
 @convert.register(np.ndarray, pd.DataFrame, cost=0.2)
 def dataframe_to_numpy(df, dshape=None, **kwargs):
     dtype = dshape_to_numpy(dshape)
@@ -51,13 +44,34 @@ def DataFrame_to_Series(x, **kwargs):
 def series_to_dataframe(x, **kwargs):
     return x.to_frame()
 
-convert.register(np.recarray, np.ndarray, cost=0.0)(identity)
-convert.register(np.ndarray, np.recarray, cost=0.0)(identity)
+
+@convert.register(np.recarray, np.ndarray, cost=0.0)
+def ndarray_to_recarray(x, **kwargs):
+    return x.view(np.recarray)
+
+@convert.register(np.ndarray, np.recarray, cost=0.0)
+def recarray_to_ndarray(x, **kwargs):
+    return x.view(np.ndarray)
+
+
+higher_precision_freqs = frozenset(('ns', 'ps', 'fs', 'as'))
 
 
 @convert.register(np.ndarray, pd.Series, cost=0.1)
-def series_to_array(s, **kwargs):
-    return np.array(s)
+def series_to_array(s, dshape=None, **kwargs):
+    dtype = datashape.to_numpy_dtype(datashape.dshape(dshape))
+    sdtype = s.dtype
+    values = s.values
+
+    # don't lose precision of datetime64 more precise than microseconds
+    if ((issubclass(sdtype.type, np.datetime64) and
+            np.datetime_data(sdtype)[0] in higher_precision_freqs)
+            or s.dtype == dtype):
+        return values
+    try:
+        return values.astype(dtype)
+    except ValueError:  # object series and record dshape, e.g., a frame row
+        return values
 
 
 @convert.register(list, np.ndarray, cost=10.0)
@@ -79,6 +93,26 @@ def numpy_chunks_to_numpy(c, **kwargs):
     return np.concatenate(list(c))
 
 
+@convert.register(chunks(np.ndarray), np.ndarray, cost=0.5)
+def numpy_to_chunks_numpy(x, chunksize=2**20, **kwargs):
+    return chunks(np.ndarray)(
+            lambda: (x[i:i+chunksize] for i in range(0, x.shape[0], chunksize)))
+
+
+@convert.register(pd.DataFrame, chunks(pd.DataFrame), cost=1.0)
+def chunks_dataframe_to_dataframe(c, **kwargs):
+    c = list(c)
+    if not c:  # empty case
+        return pd.DataFrame(columns=kwargs.get('dshape').measure.names)
+    else:
+        return pd.concat(c, axis=0, ignore_index=True)
+
+
+@convert.register(chunks(pd.DataFrame), pd.DataFrame, cost=0.5)
+def dataframe_to_chunks_dataframe(x, chunksize=2**20, **kwargs):
+    return chunks(pd.DataFrame)(
+            lambda: (x.iloc[i:i+chunksize] for i in range(0, x.shape[0], chunksize)))
+
 def ishashable(x):
     try:
         hash(x)
@@ -104,8 +138,22 @@ def iterable_to_tuple(x, **kwargs):
     return tuple(x)
 
 
+def element_of(seq):
+    """
+
+    >>> element_of([1, 2, 3])
+    1
+    >>> element_of([[1, 2], [3, 4]])
+    1
+    """
+    while isinstance(seq, list) and seq:
+        seq = seq[0]
+    return seq
+
 @convert.register(np.ndarray, list, cost=10.0)
 def list_to_numpy(seq, dshape=None, **kwargs):
+    if isinstance(element_of(seq), dict):
+        seq = list(records_to_tuples(dshape, seq))
     if (seq and isinstance(seq[0], Iterable)
             and not ishashable(seq[0])
             and not isscalar(dshape)):
@@ -144,7 +192,10 @@ def iterator_to_numpy_chunks(seq, chunksize=1024, **kwargs):
 @convert.register(chunks(pd.DataFrame), Iterator, cost=10.0)
 def iterator_to_DataFrame_chunks(seq, chunksize=1024, **kwargs):
     seq2 = partition_all(chunksize, seq)
-    first, rest = next(seq2), seq2
+    try:
+        first, rest = next(seq2), seq2
+    except StopIteration:
+        return chunks(pd.DataFrame)([])
     df = convert(pd.DataFrame, first, **kwargs)
     def _():
         yield df
@@ -158,11 +209,11 @@ def numpy_record_to_tuple(rec, **kwargs):
     return rec.tolist()
 
 
-@convert.register(chunks(np.ndarray), chunks(pd.DataFrame), cost=1.0)
+@convert.register(chunks(np.ndarray), chunks(pd.DataFrame), cost=0.5)
 def chunked_pandas_to_chunked_numpy(c, **kwargs):
     return chunks(np.ndarray)(lambda: (convert(np.ndarray, chunk, **kwargs) for chunk in c))
 
-@convert.register(chunks(pd.DataFrame), chunks(np.ndarray), cost=1.0)
+@convert.register(chunks(pd.DataFrame), chunks(np.ndarray), cost=0.5)
 def chunked_numpy_to_chunked_pandas(c, **kwargs):
     return chunks(pd.DataFrame)(lambda: (convert(pd.DataFrame, chunk, **kwargs) for chunk in c))
 
@@ -175,5 +226,17 @@ def chunked_list_to_chunked_numpy(c, **kwargs):
 def chunked_numpy_to_chunked_list(c, **kwargs):
     return chunks(list)(lambda: (convert(list, chunk, **kwargs) for chunk in c))
 
+@convert.register(chunks(Iterator), chunks(list), cost=0.1)
+def chunked_list_to_chunked_iterator(c, **kwargs):
+    return chunks(Iterator)(c.data)
+
+@convert.register(chunks(list), chunks(Iterator), cost=0.1)
+def chunked_Iterator_to_chunked_list(c, **kwargs):
+    return chunks(Iterator)(lambda: (convert(Iterator, chunk, **kwargs) for chunk in c))
+
+@convert.register(Iterator, chunks(Iterator), cost=0.1)
+def chunked_iterator_to_iterator(c, **kwargs):
+    return concat(c)
+
 
 ooc_types |= set([Iterator, Chunks])
